package handler

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/database"
	batchSeminarConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/config"
	batchSeminarTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/transport"
	batchSidangConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/config"
	batchSidangTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/transport"
	bimbinganLogConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/config"
	bimbinganLogTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/transport"
	dashboardDosbingConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/config"
	dashboardDosbingTrasport "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/transport"
	dashboardMahasiswaConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/config"
	dashboardMahasiswaTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/transport"
	dashboardTimTugasConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/config"
	dashboardTimTugasTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/transport"
	pendaftaranConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/config"
	pendaftaranTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/transport"
	pendaftaranSidSemConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/config"
	pendaftaranSidSemTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/transport"
	periodePendaftaranConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/config"
	periodePendaftaranTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/transport"
	ruanganConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/config"
	ruanganTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/transport"
	schedulingConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/config"
	schedulingTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/transport"
	systemLogConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/config"
	systemLogTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/transport"
	topicAllocationConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/config"
	topicAllocationTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/transport"
)

type Service struct {
	AdminRuanganHandler               *ruanganTransport.AdminRuanganHandler
	InternalSystemLogHandler          *systemLogTransport.InternalSystemLogHandler
	AdminSytemLogHandler              *systemLogTransport.AdminSystemLogHandler
	AdminBimbinganLogHandler          *bimbinganLogTransport.AdminBimbinganLogHandler
	AdminTopicAllocationHandler       *topicAllocationTransport.AdminTopicAllocationHandler
	BatchSeminarHandler               *batchSeminarTransport.AdminBatchSeminarHandler
	BatchSidangHandler                *batchSidangTransport.AdminBatchSidangHandler
	AdminPendaftaranSidSemHandler     *pendaftaranSidSemTransport.AdminPendaftaranSidSemHandler
	MahasiswaDashboardHandler         *dashboardMahasiswaTransport.MahasiswaDashboardHandler
	DosbingDashboardHandler           *dashboardDosbingTrasport.DosbingDashboardHandler
	AdminPendaftaranHandler           *pendaftaranTransport.AdminPendaftaranHandler
	AdminPeriodePendaftaranHandler    *periodePendaftaranTransport.AdminPeriodPendaftaranHandler
	MahasiswaBimbinganLogHandler      *bimbinganLogTransport.MahasiswaBimbinganLogHandler
	AdminDashboardTimTugasHandler     *dashboardTimTugasTransport.AdminDashboardTimTugasHandler
	DosbingBimbinganLogHandler        *bimbinganLogTransport.DosbingimbinganLogHandler
	TIMTASchedulingHandler            *schedulingTransport.TIMTASchedulingHandler
	MahasiswaPendaftaranSidSemHandler *pendaftaranSidSemTransport.MahasiswaPendaftaranSidSemHandler
	TIMTAPendaftaranSidSemHandler     *pendaftaranSidSemTransport.TIMTAPendaftaranSidSemHandler
}

func MakeHandler() *Service {
	dbWrite := database.GetInstance()
	dbRead := database.GetInstance()

	InternalSystemLogHandler := systemLogTransport.NewInternalSystemLogHandler(systemLogConfig.SystemLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminSystemLogHandler := systemLogTransport.NewAdminSystemLogHandler(systemLogConfig.SystemLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBimbinganLogHandler := bimbinganLogTransport.NewAdminBimginganLogHandler(bimbinganLogConfig.BimbinganLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminTopicAllocationHandler := topicAllocationTransport.NewAdminTopicAllocationHandler(topicAllocationConfig.TopicAllocationTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBatchSeminarHandler := batchSeminarTransport.NewAdminBatchSeminarHandler(batchSeminarConfig.BatchSeminarTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBatchSidangHandler := batchSidangTransport.NewAdminBatchSidangHandler(batchSidangConfig.BatchSidangTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	MahasiswaDashboardHandler := dashboardMahasiswaTransport.NewMahasiswaDashboardHandler(dashboardMahasiswaConfig.DashboardMahasiswaTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	DosbingDashboardHandler := dashboardDosbingTrasport.NewDosbingDashboardHandler(dashboardDosbingConfig.DashboardDosbingTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminPendaftaranHandler := pendaftaranTransport.NewAdminPendaftaranHandler(pendaftaranConfig.PendaftaranTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})

	AdminPeriodPendaftaranHandler := periodePendaftaranTransport.NewAdminPeriodPendaftaranHandler(periodePendaftaranConfig.PeriodPendaftaranConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	MahasiwaBimbinganLogHandler := bimbinganLogTransport.NewMahasiswaBimginganLogHandler(bimbinganLogConfig.BimbinganLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminDashboardTimTugasHandler := dashboardTimTugasTransport.NewDashboardTimTugasHandler(dashboardTimTugasConfig.DashboardTimTugasTransportConfig{
		DBRead: dbRead,
	})
	AdminPendaftaranSidSemHandler := pendaftaranSidSemTransport.NewAdminJadwalSeminarHandler(pendaftaranSidSemConfig.PendaftaranSidSemTransportConfig{
		DBRead:  dbRead,
		DBWrite: dbWrite,
	})

	TIMTAPendaftaranSidSemHandler := pendaftaranSidSemTransport.NewTIMTAPendaftaranSidSemHandler(pendaftaranSidSemConfig.PendaftaranSidSemTransportConfig{
		DBRead:  dbRead,
		DBWrite: dbWrite,
	})

	DosbingBimbinganLogHandler := bimbinganLogTransport.NewDosbingBimginganLogHandler(bimbinganLogConfig.BimbinganLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})

	TIMTASchedulingHandlerHandler := schedulingTransport.NewTIMTASchedulingHandler(schedulingConfig.SchedulingTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminRuanganHandler := ruanganTransport.NewAdminRuanganHandler(ruanganConfig.RuanganTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	MahasiswaPendaftaranSidSemHandler := pendaftaranSidSemTransport.NewMahasiswaPendaftaranSidSemHandler(pendaftaranSidSemConfig.PendaftaranSidSemTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})

	return &Service{
		InternalSystemLogHandler:          InternalSystemLogHandler,
		AdminSytemLogHandler:              AdminSystemLogHandler,
		AdminBimbinganLogHandler:          AdminBimbinganLogHandler,
		AdminTopicAllocationHandler:       AdminTopicAllocationHandler,
		BatchSeminarHandler:               AdminBatchSeminarHandler,
		BatchSidangHandler:                AdminBatchSidangHandler,
		AdminPendaftaranSidSemHandler:     AdminPendaftaranSidSemHandler,
		MahasiswaDashboardHandler:         MahasiswaDashboardHandler,
		DosbingDashboardHandler:           DosbingDashboardHandler,
		AdminPendaftaranHandler:           AdminPendaftaranHandler,
		AdminPeriodePendaftaranHandler:    AdminPeriodPendaftaranHandler,
		MahasiswaBimbinganLogHandler:      MahasiwaBimbinganLogHandler,
		DosbingBimbinganLogHandler:        DosbingBimbinganLogHandler,
		AdminDashboardTimTugasHandler:     AdminDashboardTimTugasHandler,
		TIMTASchedulingHandler:            TIMTASchedulingHandlerHandler,
		MahasiswaPendaftaranSidSemHandler: MahasiswaPendaftaranSidSemHandler,
		TIMTAPendaftaranSidSemHandler:     TIMTAPendaftaranSidSemHandler,
		AdminRuanganHandler:               AdminRuanganHandler,
	}
}
