package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/entity"
	"gorm.io/gorm"
)

type BatchSidangRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewBatchSidangRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *BatchSidangRepo {
	return &BatchSidangRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *BatchSidangRepo) NewBatchSidang(batchSidang entity.BatchSidang) (output entity.BatchSidang, err error) {
	batchSidang.IdBatch = uuid.New().String()
	batchSidang.Status = true

	err = repo.DBWrite.Model(&entity.BatchSidang{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.BatchSidang{}, err
	}

	result := repo.DBWrite.Create(&batchSidang)
	if result.Error != nil {
		return entity.BatchSidang{}, result.Error
	}
	return batchSidang, nil
}

func (repo *BatchSidangRepo) GetSidangBatches() (output []entity.BatchSidang, err error) {
	var batches []entity.BatchSidang
	result := repo.DBRead.Find(&batches)
	if result.Error != nil {
		return nil, err
	}
	return batches, nil
}

func (repo *BatchSidangRepo) DeleteBatch(id string) (entity.BatchSidang, error) {
	batch := entity.BatchSidang{IdBatch: id}
	err := repo.DBWrite.Delete(&batch).Error

	if err != nil {
		return batch, err
	}

	return batch, err
}

func (repo *BatchSidangRepo) GetBatchSidangCurrent() ([]entity.BatchSidang, error) {
	var batch []entity.BatchSidang
	currentTime := time.Now().Format("2006-01-02T15:04:05Z")
	result := repo.DBRead.Where(" ? BETWEEN start_date AND end_date", currentTime).Limit(1).Find(&batch)
	if result.Error != nil {
		return nil, result.Error
	}
	return batch, nil
}

func (repo *BatchSidangRepo) SetLiveBatch(id string) (entity.BatchSidang, error) {
	err := repo.DBWrite.Model(&entity.BatchSidang{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.BatchSidang{}, err
	}

	batch := entity.BatchSidang{IdBatch: id}
	err = repo.DBWrite.Model(&batch).Update("status", true).Error

	if err != nil {
		return batch, err
	}

	return batch, nil
}

func (repo *BatchSidangRepo) GetBatchSidangLive() ([]entity.BatchSidang, error) {
	var batch []entity.BatchSidang

	result := repo.DBRead.Where("status = ?", true).Limit(1).Find(&batch)
	if result.Error != nil {
		return nil, result.Error
	}

	return batch, nil
}
