package repository

import (
	entity1 "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/entity"
	entity2 "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"

	"gorm.io/gorm"
)

type DashboardTimTugasRepo struct {
	DBRead *gorm.DB
}

func NewDashboardTimTugasRepository(dbRead *gorm.DB) *DashboardTimTugasRepo {
	return &DashboardTimTugasRepo{
		DBRead: dbRead,
	}
}

func (repo *DashboardTimTugasRepo) GetDashboardStatistics(periode string) (output entity1.StatusStatistics, err error) {
	var pendaftaran []entity2.Pendaftaran
	var count int64

	result := repo.DBRead.Model(&pendaftaran).Where("id_periode = ?", periode)
	if result.Error != nil {
		return entity1.StatusStatistics{}, result.Error
	}

	countTotal := result.Count(&count)
	if countTotal.Error != nil {
		return entity1.StatusStatistics{}, countTotal.Error
	}
	var total = count

	countDiterima := result.Where("status = ?", true).Find(&pendaftaran).Count(&count)
	if countDiterima.Error != nil {
		return entity1.StatusStatistics{}, countDiterima.Error
	}
	var totalDiterima = count
	var persenDiterima float64
	if total == 0 {
		persenDiterima = 0
	} else {
		persenDiterima = float64(count) / float64(total)
	}

	countDiproses := result.Where("status IS NULL").Find(&pendaftaran).Count(&count)
	if countDiproses.Error != nil {
		return entity1.StatusStatistics{}, countDiproses.Error
	}
	var totalDiproses = count
	var persenDiproses float64
	if total == 0 {
		persenDiproses = 0
	} else {
		persenDiproses = float64(count) / float64(total)
	}

	countDitolak := result.Where("status = ?", false).Find(&pendaftaran).Count(&count)
	if countDitolak.Error != nil {
		return entity1.StatusStatistics{}, countDitolak.Error
	}
	var totalDitolak = count
	var persenDitolak float64
	if total == 0 {
		persenDitolak = 0
	} else {
		persenDitolak = float64(count) / float64(total)
	}

	statistics := entity1.StatusStatistics{
		Diterima:       totalDiterima,
		PersenDiterima: persenDiterima,
		Diproses:       totalDiproses,
		PersenDiproses: persenDiproses,
		Ditolak:        totalDitolak,
		PersenDitolak:  persenDitolak,
	}

	return statistics, nil
}

func (repo *DashboardTimTugasRepo) GetDaftarMahasiswa(periode string) (output []entity1.DataPendaftaranMahasiswa, err error) {
	var pendaftaran []entity1.DataPendaftaranMahasiswa

	var pendaftarans []entity1.Pendaftaran

	result := repo.DBRead.Where("id_periode = ?", periode).Find(&pendaftarans)
	if result.Error != nil {
		return nil, result.Error
	}

	for _, pendaftaran := range pendaftarans {
		var mahasiswa entity1.Mahasiswa
		var topik entity1.Topik

		repo.DBRead.Where("id_mahasiswa = ?", pendaftaran.IdMahasiswa).First(&mahasiswa)
		repo.DBRead.Where("id = ?", pendaftaran.IdTopik).First(&topik)

		output = append(output, entity1.DataPendaftaranMahasiswa{
			NIM:    mahasiswa.NIM,
			Nama:   mahasiswa.Name,
			Topik:  topik.Judul,
			Status: pendaftaran.Status,
		})
	}

	if result.Error != nil {
		return nil, result.Error
	}

	return pendaftaran, nil
}

func (repo *DashboardTimTugasRepo) GetPeriodData() (output []entity1.PeriodStats, err error) {
	var periods []entity1.PeriodPendaftaran

	result := repo.DBRead.Find(&periods).Order("start_date DESC")
	if result.Error != nil {
		return nil, result.Error
	}

	var periodStats []entity1.PeriodStats
	for _, period := range periods {
		var pendaftarans entity1.Pendaftaran
		var count int64

		result := repo.DBRead.Where("id_periode = ? & status = ?", period.IDPeriod, true).Find(&pendaftarans).Count(&count)
		if result.Error != nil {
			return nil, result.Error
		}

		periodStats = append(periodStats, entity1.PeriodStats {
			Period: period,
			NumMahasiswa: count,
		})
	}

	return periodStats, nil
}
