package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"

	"gorm.io/gorm"
)

type PendaftaranSidSemRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewPendaftaranSidSemRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *PendaftaranSidSemRepo {
	return &PendaftaranSidSemRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *PendaftaranSidSemRepo) NewPendaftaranSidSem(jadwal entity.PendaftaranSidSem, berkas []entity.BerkasSidSem) (output entity.PendaftaranSidSemBerkas, err error) {
	result := repo.DBWrite.Create(&jadwal)
	if result.Error != nil {
		return entity.PendaftaranSidSemBerkas{}, result.Error
	}

	berkasRepo := NewBerkasSidsemRepository(repo.DBWrite, repo.DBRead)
	berkas, err = berkasRepo.NewBatchBerkas(jadwal.ID, berkas)
	if err != nil {
		return
	}

	pendaftaranSidSemBerkas := entity.PendaftaranSidSemBerkas{
		ID:              jadwal.ID,
		PendaftaranTAID: jadwal.PendaftaranTAID,
		Tipe:            jadwal.Tipe,
		Ditolak:         jadwal.Ditolak,
		Lulus:           jadwal.Lulus,
		JudulProposal:   jadwal.JudulProposal,
		Deskripsi:       jadwal.Deskripsi,
		WaktuMulai:      jadwal.WaktuMulai,
		WaktuSelesai:    jadwal.WaktuSelesai,
		IdPeriode:       jadwal.IdPeriode,
		RuanganID:       nil,
		BerkasSidSem:    berkas,
	}

	return pendaftaranSidSemBerkas, nil
}

func (repo *PendaftaranSidSemRepo) GetPendaftaranByPeriode(idPeriode string, offset, size int) (output []entity.PendaftaranSidSemRequest, err error) {
	var pendaftaransidsem []models.PendaftaranSidSem
	var pendaftaranTA models.PendaftaranTA
	var mahasiswa models.Pengguna
	var dataJadwal []entity.PendaftaranSidSemRequest
	result := repo.DBRead.Limit(size).Offset(offset).Where("id_periode = ?", idPeriode).Find(&pendaftaransidsem)

	for _, jadwal := range pendaftaransidsem {
		res := repo.DBRead.Where("id = ?", jadwal.PendaftaranTAID).Limit(1).Find(&pendaftaranTA)
		if res.Error != nil {
			return nil, res.Error
		}

		res = repo.DBRead.Where("id = ?", pendaftaranTA.IdMahasiswa).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}

		data := entity.PendaftaranSidSemRequest{
			ID:           jadwal.ID,
			Nim:          mahasiswa.NIM,
			Nama:         mahasiswa.Nama,
			WaktuMulai:   jadwal.WaktuMulai,
			WaktuSelesai: jadwal.WaktuSelesai,
			Tipe:         jadwal.Tipe,
			Link:         jadwal.Link,
		}

		dataJadwal = append(dataJadwal, data)

	}

	if result.Error != nil {
		return nil, err
	}
	return dataJadwal, nil
}

func (repo *PendaftaranSidSemRepo) GetPendaftaran(offset, size int) (output []entity.RekapPendaftaranRequest, err error) {
	var pendaftaransidsem []models.PendaftaranSidSem
	var pendaftaranTA models.PendaftaranTA
	var mahasiswa models.Pengguna
	var dosbing models.Pengguna
	var dataJadwal []entity.RekapPendaftaranRequest
	result := repo.DBRead.Find(&pendaftaransidsem).Limit(size).Offset(offset)

	for _, jadwal := range pendaftaransidsem {
		print(jadwal.ID)
	}
	for _, jadwal := range pendaftaransidsem {
		res := repo.DBRead.Where("id = ?", jadwal.PendaftaranTAID).Limit(1).Find(&pendaftaranTA)
		if res.Error != nil {
			return nil, res.Error
		}

		res = repo.DBRead.Where("id = ?", pendaftaranTA.IdMahasiswa).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}

		res = repo.DBRead.Where("id = ?", pendaftaranTA.IdDosen).Find(&dosbing)

		data := entity.RekapPendaftaranRequest{
			ID:          jadwal.ID,
			Nim:         mahasiswa.NIM,
			IdMahasiswa: mahasiswa.ID,
			Nama:        mahasiswa.Nama,
			Dosbing:     dosbing.Nama,
			Tipe:        jadwal.Tipe,
			Ditolak:     jadwal.Ditolak,
		}

		dataJadwal = append(dataJadwal, data)

	}
	if result.Error != nil {
		return nil, err
	}
	return dataJadwal, nil
}

func (repo *PendaftaranSidSemRepo) GetPendaftaranSeminarById(id string) (output entity.SeminarPendaftaranSidSemRequest, err error) {
	var pendaftaransem models.PendaftaranSidSem
	var pendaftaranTA models.PendaftaranTA
	var mahasiswa models.Pengguna
	var dosbing models.Pengguna
	var dosuji1 models.Pengguna
	var pengujisem models.PengujiSidsem
	var dataJadwal entity.SeminarPendaftaranSidSemRequest
	var topik models.Topik
	var ruang models.Ruangan

	res := repo.DBRead.Limit(1).Where("id = ?", id).Find(&pendaftaransem)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaransem.PendaftaranTAID).Limit(1).Find(&pendaftaranTA)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdMahasiswa).Find(&mahasiswa)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdDosen).Find(&dosbing)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("sidsem_id = ?", pendaftaransem.ID).Find(&pengujisem)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pengujisem.DosenID).Find(&dosuji1)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdTopik).Find(&topik)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaransem.RuanganID).Find(&ruang)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	dataJadwal = entity.SeminarPendaftaranSidSemRequest{
		ID:              pendaftaransem.ID,
		Nim:             mahasiswa.NIM,
		Nama:            mahasiswa.Nama,
		Email:           mahasiswa.Email,
		Topik:           topik.Judul,
		Deskripsi:       topik.Deskripsi,
		DosenPembimbing: dosbing.Nama,
		DosenPenguji:    dosuji1.Nama,
		WaktuMulai:      pendaftaransem.WaktuMulai,
		WaktuSelesai:    pendaftaransem.WaktuSelesai,
		Tipe:            pendaftaransem.Tipe,
		Link:            pendaftaransem.Link,
		Ruang:           ruang.Nama,
	}

	return dataJadwal, nil
}

func (repo *PendaftaranSidSemRepo) GetPendaftaranSidangById(id string) (output entity.SidangPendaftaranSidSemRequest, err error) {
	var pendaftaransem models.PendaftaranSidSem
	var pendaftaranTA models.PendaftaranTA
	var mahasiswa models.Pengguna
	var dosbing models.Pengguna
	var dosuji1 models.Pengguna
	var dosuji2 models.Pengguna
	var pengujisem []models.PengujiSidsem
	var dataJadwal entity.SidangPendaftaranSidSemRequest
	var topik models.Topik
	var ruang models.Ruangan

	res := repo.DBRead.Limit(1).Where("id = ?", id).Find(&pendaftaransem)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaransem.PendaftaranTAID).Limit(1).Find(&pendaftaranTA)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdMahasiswa).Find(&mahasiswa)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdDosen).Find(&dosbing)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("sidsem_id = ?", pendaftaransem.ID).Limit(2).Find(&pengujisem)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaranTA.IdTopik).Find(&topik)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	res = repo.DBRead.Where("id = ?", pendaftaransem.RuanganID).Find(&ruang)
	if res.Error != nil {
		return dataJadwal, res.Error
	}

	for _, penguji := range pengujisem {
		if dosuji1 == (models.Pengguna{}) {
			res = repo.DBRead.Where("id = ?", penguji.DosenID).Find(&dosuji1)
			if res.Error != nil {
				return dataJadwal, res.Error
			}
		} else {
			res = repo.DBRead.Where("id = ?", penguji.DosenID).Find(&dosuji2)
			if res.Error != nil {
				return dataJadwal, res.Error
			}
		}
	}

	dataJadwal = entity.SidangPendaftaranSidSemRequest{
		ID:              pendaftaransem.ID,
		Nim:             mahasiswa.NIM,
		Nama:            mahasiswa.Nama,
		Email:           mahasiswa.Email,
		Topik:           topik.Judul,
		Deskripsi:       topik.Deskripsi,
		DosenPembimbing: dosbing.Nama,
		DosenPenguji:    dosuji1.Nama,
		DosenPenguji2:   dosuji2.Nama,
		WaktuMulai:      pendaftaransem.WaktuMulai,
		WaktuSelesai:    pendaftaransem.WaktuSelesai,
		Tipe:            pendaftaransem.Tipe,
		Link:            pendaftaransem.Link,
		Ruang:           ruang.Nama,
	}

	return dataJadwal, nil
}

func (repo *PendaftaranSidSemRepo) EditRuangan(id string, id_ruangan string) (entity.PendaftaranSidSem, error) {
	var pendaftaran entity.PendaftaranSidSem

	err := repo.DBWrite.Where("id = ?", id).First(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}
	pendaftaran.RuanganID = &id_ruangan
	err = repo.DBWrite.Save(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	return pendaftaran, nil
}

func (repo *PendaftaranSidSemRepo) EditDosenPengujiSidang(input entity.EditDosenPengujiSidangRequest) ([]models.PengujiSidsem, error) {
	var penguji []models.PengujiSidsem

	err := repo.DBWrite.Where("sidsem_id = ?", input.IdSidSem).Find(&penguji).Error
	if err != nil {
		return penguji, err
	}

	if len(penguji) == 0 {
		penguji1 := models.PengujiSidsem{
			ID:       uuid.New().String(),
			SidsemID: input.IdSidSem,
			DosenID:  input.IdDosen1,
		}

		penguji2 := models.PengujiSidsem{
			ID:       uuid.New().String(),
			SidsemID: input.IdSidSem,
			DosenID:  input.IdDosen2,
		}

		err = repo.DBWrite.Create(&penguji1).Error
		if err != nil {
			return penguji, err
		}

		err = repo.DBWrite.Create(&penguji2).Error
		if err != nil {
			return penguji, err
		}
	} else if len(penguji) == 1 {
		penguji[0].DosenID = input.IdDosen1

		err = repo.DBWrite.Save(&penguji[0]).Error
		if err != nil {
			return penguji, err
		}
		penguji2 := models.PengujiSidsem{
			ID:       uuid.New().String(),
			SidsemID: input.IdSidSem,
			DosenID:  input.IdDosen2,
		}
		err = repo.DBWrite.Create(&penguji2).Error
		if err != nil {
			return penguji, err
		}
	} else {
		penguji[0].DosenID = input.IdDosen1
		penguji[1].DosenID = input.IdDosen2

		err = repo.DBWrite.Save(&penguji[0]).Error
		if err != nil {
			return penguji, err
		}

		err = repo.DBWrite.Save(&penguji[1]).Error
		if err != nil {
			return penguji, err
		}
	}

	return penguji, nil
}

func (repo *PendaftaranSidSemRepo) EditDosenPengujiSeminar(input entity.EditDosenPengujiSeminarRequest) ([]models.PengujiSidsem, error) {
	var penguji []models.PengujiSidsem

	err := repo.DBWrite.Where("sidsem_id = ?", input.IdSidSem).Limit(1).Find(&penguji).Error
	if err != nil {
		return penguji, err
	}

	if len(penguji) == 0 {
		penguji1 := models.PengujiSidsem{
			ID:       uuid.New().String(),
			SidsemID: input.IdSidSem,
			DosenID:  input.IdDosen1,
		}

		err = repo.DBWrite.Create(&penguji1).Error
		if err != nil {
			return penguji, err
		}
	} else {
		penguji[0].DosenID = input.IdDosen1

		err = repo.DBWrite.Save(&penguji[0]).Error
		if err != nil {
			return penguji, err
		}
	}

	return penguji, nil
}

func (repo *PendaftaranSidSemRepo) EditWaktu(input entity.EditWaktuRequest) (models.PendaftaranSidSem, error) {
	var pendaftaran models.PendaftaranSidSem

	err := repo.DBWrite.Where("id = ?", input.Id).Limit(1).Find(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	if input.WaktuMulai != (time.Time{}) {
		pendaftaran.WaktuMulai = input.WaktuMulai
	}

	if input.WaktuSelesai != (time.Time{}) {
		pendaftaran.WaktuSelesai = input.WaktuSelesai
	}

	err = repo.DBWrite.Save(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	return pendaftaran, nil
}

func (repo *PendaftaranSidSemRepo) DeletePendaftaran(id string) (entity.PendaftaranSidSem, error) {
	pendaftaran := entity.PendaftaranSidSem{ID: id}
	err := repo.DBWrite.Delete(&pendaftaran).Error

	if err != nil {
		return pendaftaran, err
	}

	return pendaftaran, err
}

func (repo *PendaftaranSidSemRepo) EditLulus(input entity.EditLulusRequest) (models.PendaftaranSidSem, error) {
	var pendaftaran models.PendaftaranSidSem

	err := repo.DBWrite.Where("id = ?", input.Id).Limit(1).Find(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	pendaftaran.Lulus = input.Lulus

	err = repo.DBWrite.Save(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	return pendaftaran, nil
}

func (repo *PendaftaranSidSemRepo) EditDitolak(input entity.EditDitolakRequest) (entity.PendaftaranSidSem, error) {
	var pendaftaran entity.PendaftaranSidSem

	err := repo.DBWrite.Where("id = ?", input.Id).Find(&pendaftaran).Limit(1).Error
	if err != nil {
		return pendaftaran, err
	}

	pendaftaran.Ditolak = input.Ditolak

	err = repo.DBWrite.Save(&pendaftaran).Error
	if err != nil {
		return pendaftaran, err
	}

	return pendaftaran, nil
}

func (repo *PendaftaranSidSemRepo) GetNameUntilTopikPlaceholders(idMahasiswa string) (entity.NameTopik, error) {
	var nameTopik entity.NameTopik

	query := `SELECT p.nama, p.nim, pt.jalur_pilihan, t.judul from pengguna p JOIN pendaftaran_ta pt ON pt.id_mahasiswa = p.id JOIN topik t ON pt.id_topik = t.id WHERE id_mahasiswa = ?;`

	if err := repo.DBRead.Raw(query, idMahasiswa).Scan(&nameTopik).Error; err != nil {
		return entity.NameTopik{}, err
	}

	return nameTopik, nil
}

func (repo *PendaftaranSidSemRepo) GetDosbingNamePlaceholder(idMahasiswa string) (entity.DosbingName, error) {
	var dosbingName entity.DosbingName

	query := ` SELECT p.nama FROM pengguna p JOIN dosen_bimbingan db ON p.id = db.dosen_id JOIN pendaftaran_ta pt ON db.pendaftaran_id = pt.id WHERE pt.id_mahasiswa = ? AND pt.status = true;`

	if err := repo.DBRead.Raw(query, idMahasiswa).Scan(&dosbingName).Error; err != nil {
		return entity.DosbingName{}, err
	}

	return dosbingName, nil
}

func (repo *PendaftaranSidSemRepo) GetPendaftaranTaIdByMahasiswa(idMahasiswa string) (string, error) {
	var pendaftaranTaID string

	query := `SELECT id FROM pendaftaran_ta WHERE id_mahasiswa = ?`

	if err := repo.DBRead.Raw(query, idMahasiswa).Scan(&pendaftaranTaID).Error; err != nil {
		return "", err
	}

	return pendaftaranTaID, nil
}

func (repo *PendaftaranSidSemRepo) IsRegisteredSidsem(idMahasiswa string, tipe string) (string, error) {
	var pendaftaranSidSemId string

	query := `SELECT ps.id FROM pendaftaran_sidsem ps JOIN pendaftaran_ta pt ON ps.pendaftaran_ta_id = pt.id JOIN pengguna p ON pt.id_mahasiswa = p.id WHERE p.id = ? AND ps.tipe = ?`

	if err := repo.DBRead.Raw(query, idMahasiswa, tipe).Scan(&pendaftaranSidSemId).Error; err != nil {
		return "", err
	}

	return pendaftaranSidSemId, nil
}

func (repo *PendaftaranSidSemRepo) GetInfoTopikMahasiswa(idMahasiswa string) (entity.InfoTopikMahasiswa, error) {
	var infoTopik entity.InfoTopikMahasiswa

	query := `SELECT pt.id_mahasiswa, pg.nama, pg.email, pt.jalur_pilihan , pt.waktu_pengiriman,
                     t.judul, t.deskripsi
              FROM pendaftaran_ta pt
              JOIN pengguna pg ON pt.id_mahasiswa = pg.id
              JOIN topik t ON pt.id_topik = t.id
              WHERE pt.id_mahasiswa = ?`

	if err := repo.DBRead.Raw(query, idMahasiswa).Scan(&infoTopik).Error; err != nil {
		return entity.InfoTopikMahasiswa{}, err
	}

	return infoTopik, nil
}

func (repo *PendaftaranSidSemRepo) GetRemainingDetailSidSem(idPendaftaranSidSem string) (entity.RemainingDetailSidSem, error) {
	var remDet entity.RemainingDetailSidSem

	result := repo.DBRead.Table("pendaftaran_sidsem ps").
		Select("ps.tipe, ps.waktu_mulai, ps.ruangan_id, ps.ditolak").
		Where("ps.id = ?", idPendaftaranSidSem).
		First(&remDet)

	if result.Error == gorm.ErrRecordNotFound {
		return entity.RemainingDetailSidSem{}, nil
	}

	if result.Error != nil {
		return entity.RemainingDetailSidSem{}, result.Error
	}

	return remDet, nil
}
