package usecase

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"
)

type PendaftaranSidSemUsecase interface {
	AddPendaftaran(pendaftaranSidSemBerkas entity.PendaftaranSidSemBerkasReq) (output entity.PendaftaranSidSemBerkas, err error)
	GetPendaftaranByPeriode(idPeriode string, offset, size int) (output []entity.PendaftaranSidSemRequest, err error)
	GetPendaftaranSeminarById(id string) (output entity.SeminarPendaftaranSidSemRequest, err error)
	GetPendaftaranSidangById(id string) (output entity.SidangPendaftaranSidSemRequest, err error)
	EditRuangan(id string, id_ruangan string) (entity.PendaftaranSidSem, error)
	EditDosenPengujiSidang(input entity.EditDosenPengujiSidangRequest) ([]models.PengujiSidsem, error)
	EditDosenPengujiSeminar(input entity.EditDosenPengujiSeminarRequest) ([]models.PengujiSidsem, error)
	EditWaktu(input entity.EditWaktuRequest) (models.PendaftaranSidSem, error)
	DeletePendaftaran(id string) (entity.PendaftaranSidSem, error)
	EditLulus(input entity.EditLulusRequest) (models.PendaftaranSidSem, error)
	EditDitolak(input entity.EditDitolakRequest) (entity.PendaftaranSidSem, error)
	GetPlaceholders(idMahasiswa string) (entity.Placeholders, error)
	GetPendaftaran(offset, size int) (output []entity.RekapPendaftaranRequest, err error)
	IsRegisteredSidsem(idMahasiswa string, tipe string) (bool, error)
	GetSidSemDetailByMahasiswaId(idMahasiswa string, tipe string) (entity.DetailSidSemByMahasiswa, error)
}

type PendaftaranSidSemUc struct {
	pendaftaransidsemrepo PendaftaranSidSemRepository
}

func NewPendaftaranSidSemUseCase(pendaftaransidsemrepo PendaftaranSidSemRepository) *PendaftaranSidSemUc {
	return &PendaftaranSidSemUc{
		pendaftaransidsemrepo: pendaftaransidsemrepo,
	}
}

func (uc *PendaftaranSidSemUc) AddPendaftaran(pendaftaranSidSemBerkas entity.PendaftaranSidSemBerkasReq) (output entity.PendaftaranSidSemBerkas, err error) {
	var nullTime time.Time
	pendaftaranTAID, err := uc.pendaftaransidsemrepo.GetPendaftaranTaIdByMahasiswa(pendaftaranSidSemBerkas.IDMahasiswa)
	if err != nil {
		return entity.PendaftaranSidSemBerkas{}, err
	}
	berkas := pendaftaranSidSemBerkas.BerkasSidSem
	pendaftaranSidSem := entity.PendaftaranSidSem{
		ID:              uuid.New().String(),
		PendaftaranTAID: pendaftaranTAID,
		Tipe:            pendaftaranSidSemBerkas.Tipe,
		Ditolak:         false,
		Lulus:           false,
		JudulProposal:   pendaftaranSidSemBerkas.JudulProposal,
		Deskripsi:       pendaftaranSidSemBerkas.Deskripsi,
		RuanganID:       nil,
		WaktuMulai:      nullTime,
		WaktuSelesai:    nullTime,
	}
	newPendaftaran, err := uc.pendaftaransidsemrepo.NewPendaftaranSidSem(pendaftaranSidSem, berkas)

	if err != nil {
		return newPendaftaran, err
	}

	return newPendaftaran, nil

}

func (uc *PendaftaranSidSemUc) GetPendaftaranByPeriode(idPeriode string, offset, size int) (output []entity.PendaftaranSidSemRequest, err error) {
	return uc.pendaftaransidsemrepo.GetPendaftaranByPeriode(idPeriode, offset, size)
}

func (uc *PendaftaranSidSemUc) GetPendaftaran(offset, size int) (output []entity.RekapPendaftaranRequest, err error) {
	return uc.pendaftaransidsemrepo.GetPendaftaran(offset, size)
}

func (uc *PendaftaranSidSemUc) GetPendaftaranSidangById(id string) (entity.SidangPendaftaranSidSemRequest, error) {
	return uc.pendaftaransidsemrepo.GetPendaftaranSidangById(id)
}

func (uc *PendaftaranSidSemUc) GetPendaftaranSeminarById(id string) (entity.SeminarPendaftaranSidSemRequest, error) {
	return uc.pendaftaransidsemrepo.GetPendaftaranSeminarById(id)
}

func (uc *PendaftaranSidSemUc) EditRuangan(id string, id_ruangan string) (entity.PendaftaranSidSem, error) {
	return uc.pendaftaransidsemrepo.EditRuangan(id, id_ruangan)
}

func (uc *PendaftaranSidSemUc) EditDosenPengujiSidang(input entity.EditDosenPengujiSidangRequest) ([]models.PengujiSidsem, error) {
	return uc.pendaftaransidsemrepo.EditDosenPengujiSidang(input)
}

func (uc *PendaftaranSidSemUc) EditDosenPengujiSeminar(input entity.EditDosenPengujiSeminarRequest) ([]models.PengujiSidsem, error) {
	return uc.pendaftaransidsemrepo.EditDosenPengujiSeminar(input)
}

func (uc *PendaftaranSidSemUc) EditWaktu(input entity.EditWaktuRequest) (models.PendaftaranSidSem, error) {
	return uc.pendaftaransidsemrepo.EditWaktu(input)
}

func (uc *PendaftaranSidSemUc) DeletePendaftaran(id string) (entity.PendaftaranSidSem, error) {
	return uc.pendaftaransidsemrepo.DeletePendaftaran(id)
}

func (uc *PendaftaranSidSemUc) EditLulus(input entity.EditLulusRequest) (models.PendaftaranSidSem, error) {
	return uc.pendaftaransidsemrepo.EditLulus(input)
}

func (uc *PendaftaranSidSemUc) EditDitolak(input entity.EditDitolakRequest) (entity.PendaftaranSidSem, error) {
	return uc.pendaftaransidsemrepo.EditDitolak(input)
}

func (uc *PendaftaranSidSemUc) IsRegisteredSidsem(idMahasiswa string, tipe string) (bool, error) {
	result, err := uc.pendaftaransidsemrepo.IsRegisteredSidsem(idMahasiswa, tipe)
	if err != nil {
		return false, err
	}

	if result != "" {
		return true, err
	}

	return false, err
}

func (uc *PendaftaranSidSemUc) GetPlaceholders(idMahasiswa string) (entity.Placeholders, error) {
	nameTopik, err := uc.pendaftaransidsemrepo.GetNameUntilTopikPlaceholders(idMahasiswa)
	if err != nil {
		return entity.Placeholders{}, err
	}

	dosbingName, err := uc.pendaftaransidsemrepo.GetDosbingNamePlaceholder(idMahasiswa)
	if err != nil {
		return entity.Placeholders{}, err
	}

	placeholders := entity.Placeholders{
		Nama:         nameTopik.Nama,
		Nim:          nameTopik.Nim,
		JalurPilihan: nameTopik.JalurPilihan,
		Judul:        nameTopik.Judul,
		Dosbing:      dosbingName.Nama,
	}

	return placeholders, nil
}

func (uc *PendaftaranSidSemUc) GetSidSemDetailByMahasiswaId(idMahasiswa string, tipe string) (entity.DetailSidSemByMahasiswa, error) {
	var namaRuangan string

	infoTopik, err := uc.pendaftaransidsemrepo.GetInfoTopikMahasiswa(idMahasiswa)
	if err != nil {
		return entity.DetailSidSemByMahasiswa{}, err
	}

	pendaftaranSidSemId, err := uc.pendaftaransidsemrepo.IsRegisteredSidsem(idMahasiswa, tipe)
	if err != nil {
		return entity.DetailSidSemByMahasiswa{}, err
	}

	dosbingName, err := uc.pendaftaransidsemrepo.GetDosbingNamePlaceholder(idMahasiswa)
	if err != nil {
		return entity.DetailSidSemByMahasiswa{}, err
	}

	remDet, err := uc.pendaftaransidsemrepo.GetRemainingDetailSidSem(pendaftaranSidSemId)
	if err != nil {
		return entity.DetailSidSemByMahasiswa{}, err
	}

	if remDet.RuanganID != "" {
		// Get ruangan name
	} else {
		namaRuangan = "Belum Ada"
	}

	detailSidSemByMahasiswa := entity.DetailSidSemByMahasiswa{
		IDMahasiswa:  infoTopik.IDMahasiswa,
		Nama:         infoTopik.Nama,
		Email:        infoTopik.Email,
		JalurPilihan: infoTopik.JalurPilihan,
		Judul:        infoTopik.Judul,
		Deskripsi:    infoTopik.Deskripsi,
		DosbingName:  dosbingName.Nama,
		Tipe:         remDet.Tipe,
		WaktuMulai:   remDet.WaktuMulai,
		NamaRuangan:  namaRuangan,
		Ditolak:      remDet.Ditolak,
	}

	return detailSidSemByMahasiswa, nil

}
