package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TIMTAPendaftaranSidSemHandler struct {
	pendaftaransidsemUsecase usecase.PendaftaranSidSemUsecase
}

func NewTIMTAPendaftaranSidSemHandler(cfg config.PendaftaranSidSemTransportConfig) *TIMTAPendaftaranSidSemHandler {
	pendaftaransidsemRepository := repository.NewPendaftaranSidSemRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaransidsemUsecase := usecase.NewPendaftaranSidSemUseCase(pendaftaransidsemRepository)

	return &TIMTAPendaftaranSidSemHandler{
		pendaftaransidsemUsecase: pendaftaransidsemUsecase,
	}

}

func (t *TIMTAPendaftaranSidSemHandler) MountTIMTA(group *echo.Group) {
	group.GET("/pendaftaran-sidsem", t.GetPendaftaran)
	group.PATCH("/pendaftaran-ditolak", t.EditDitolak)

}

func (t *TIMTAPendaftaranSidSemHandler) GetPendaftaran(c echo.Context) error {
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		size = 10
	}
	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		offset = 0
	}
	jadwal, err := t.pendaftaransidsemUsecase.GetPendaftaran(size, offset)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", jadwal))
}


func (t *TIMTAPendaftaranSidSemHandler) EditDitolak(c echo.Context) error {
	request := new(entity.EditDitolakRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditDitolak(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}
