package repository

import (
	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gorm.io/gorm"
)

type TopicAllocationRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewTopicAllocationRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *TopicAllocationRepo {
	return &TopicAllocationRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *TopicAllocationRepo) NewTopic(topic entity.Topik) (output entity.Topik, err error) {
	topic.Id = uuid.New().String()

	result := repo.DBWrite.Create(&topic)
	if result.Error != nil {
		return entity.Topik{}, result.Error
	}

	return topic, nil
}

func (repo *TopicAllocationRepo) GetTopic(page int, limit int, search string, id string, periode string) (output []entity.Topik, err error) {
	var topics []entity.Topik

	result := repo.DBRead.Find(&topics).Where("Periode = ?", periode)
	if search != "" {
		result = result.Where("Judul LIKE ?", "%"+search+"%").Find(&topics)
		if result.Error != nil {
			return nil, result.Error
		}
	}

	if id != "" {
		result = result.Where("IdDosen = ?", id).Find(&topics)
		if result.Error != nil {
			return nil, result.Error
		}
	}

	if page != 0 {
		result = result.Offset((page - 1) * limit).Find(&topics)
		if result.Error != nil {
			return nil, result.Error
		}
	}

	if limit != 0 {
		result = result.Limit(limit).Find(&topics)
		if result.Error != nil {
			return nil, result.Error
		}
	}

	return topics, nil
}

func (repo *TopicAllocationRepo) GetTopicByID(ID string) (output entity.Topik, err error) {
	var topic entity.Topik

	result := repo.DBRead.Where("Id = ?", ID).First(&topic)
	if result.Error != nil {
		return entity.Topik{}, result.Error
	}

	return topic, nil
}

func (repo *TopicAllocationRepo) UpdateTopic(topic entity.Topik) (output entity.Topik, err error) {
	result := repo.DBWrite.Save(&topic)

	if result.Error != nil {
		return entity.Topik{}, result.Error
	}

	return topic, nil
}

func (repo *TopicAllocationRepo) DeleteTopic(ID string) (err error) {
	var topic entity.Topik

	result := repo.DBWrite.Where("Id = ?", ID).Delete(&topic)

	if result.Error != nil {
		return result.Error
	}

	return nil
}
